@@ -2,16 +2,14 @@
 #
 # Workbook - A class for writing the Excel XLSX Workbook file.
 #
-# Copyright 2013-2018, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2019, John McNamara, jmcnamara@cpan.org
 #
 
 # Standard packages.
-import sys
 import re
 import os
 import operator
 import time
-import warnings
 from warnings import warn
 from datetime import datetime
 from zipfile import ZipFile, ZipInfo, ZIP_DEFLATED
@@ -344,7 +342,6 @@ class Workbook(xmlwriter.XMLwriter):
 
         if tab_ratio < 0 or tab_ratio > 100:
             warn("Tab ratio '%d' outside: 0 <= tab_ratio <= 100" % tab_ratio)
-            tab_ratio = 100
         else:
             self.tab_ratio = int(tab_ratio * 10)
 
@@ -658,6 +655,7 @@ class Workbook(xmlwriter.XMLwriter):
         for os_filename, xml_filename, is_binary in xml_files:
             if self.in_memory:
 
+                # Set sub-file timestamp to Excel's timestamp of 1/1/1980.
                 zipinfo = ZipInfo(xml_filename, (1980, 1, 1, 0, 0, 0))
 
                 # Copy compression type from parent ZipFile.
@@ -671,7 +669,8 @@ class Workbook(xmlwriter.XMLwriter):
             else:
                 # The files are tempfiles.
 
-                timestamp = time.mktime((1980, 1, 1, 0, 0, 0, 0, 0, 0))
+                # Set sub-file timestamp to Excel's timestamp of 1/1/1980.
+                timestamp = time.mktime((1980, 1, 1, 0, 0, 0, 0, 0, -1))
                 os.utime(os_filename, (timestamp, timestamp))
 
                 xlsx_file.write(os_filename, xml_filename)
@@ -682,10 +681,6 @@ class Workbook(xmlwriter.XMLwriter):
     def _add_sheet(self, name, worksheet_class=None):
         # Utility for shared code in add_worksheet() and add_chartsheet().
 
-        if worksheet_class is None:
-            raise ValueError(
-                "_add_sheet() must define 'worksheet_class'")
-
         if worksheet_class:
             worksheet = worksheet_class()
         else:
@@ -862,7 +857,21 @@ class Workbook(xmlwriter.XMLwriter):
 
             # Check if num_format is an index to a built-in number format.
             if not isinstance(num_format, str_types):
-                xf_format.num_format_index = int(num_format)
+                num_format = int(num_format)
+
+                # Number format '0' is indexed as 1 in Excel.
+                if num_format == 0:
+                    num_format = 1
+
+                xf_format.num_format_index = num_format
+                continue
+            elif num_format == '0':
+                # Number format '0' is indexed as 1 in Excel.
+                xf_format.num_format_index = 1
+                continue
+            elif num_format == 'General':
+                # The 'General' format has an number format index of 0.
+                xf_format.num_format_index = 0
                 continue
 
             if num_format in num_formats:
@@ -1043,8 +1052,6 @@ class Workbook(xmlwriter.XMLwriter):
         chart_ref_id = 0
         image_ref_id = 0
         drawing_id = 0
-        x_dpi = 96
-        y_dpi = 96
 
         for sheet in self.worksheets():
             chart_count = len(sheet.charts)
@@ -1156,16 +1163,9 @@ class Workbook(xmlwriter.XMLwriter):
         marker4 = unpack('<L', data[:4])[0]
         marker5 = (unpack('4s', data[40:44]))[0]
 
-        if sys.version_info < (2, 6, 0):
-            # Python 2.5/Jython.
-            png_marker = 'PNG'
-            bmp_marker = 'BM'
-            emf_marker = ' EMF'
-        else:
-            # Eval the binary literals for Python 2.5/Jython compatibility.
-            png_marker = eval("b'PNG'")
-            bmp_marker = eval("b'BM'")
-            emf_marker = eval("b' EMF'")
+        png_marker = b'PNG'
+        bmp_marker = b'BM'
+        emf_marker = b' EMF'
 
         if marker1 == png_marker:
             self.image_types['png'] = True
